#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from census_data_downloader.core.tables import BaseTableConfig
from census_data_downloader.core.decorators import register


@register
class OccupantsDownloader(BaseTableConfig):
    PROCESSED_TABLE_NAME = "occupants"
    UNIVERSE = "occupied housing unts"
    RAW_TABLE_NAME = "B25014"
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': "universe",
        '002': "owner_occupied",
        '003': "owner_occupied_050_or_less",
        '004': "owner_occupied_051_to_100",
        '005': "owner_occupied_101_to_150",
        '006': "owner_occupied_151_to_200",
        '007': "owner_occupied_201_or_more",
        '008': "renter_occupied",
        '009': "renter_occupied_050_or_less",
        '010': "renter_occupied_051_to_100",
        '011': "renter_occupied_101_to_150",
        '012': "renter_occupied_151_to_200",
        '013': "renter_occupied_201_or_more"
    })



#! /usr/bin/env python
# -*- coding: utf-8 -*
"""
A base class that governs how to download and process tables from a Census API table.
"""
import os
import logging
import pathlib
from . import geotypes
from . import decorators
logger = logging.getLogger(__name__)


class BaseTableConfig(object):
    """
    Configures how to download and process tables from the Census API.
    """
    THIS_DIR = pathlib.Path(__file__).parent
    PARENT_DIR = THIS_DIR.parent
    # All available years
    YEAR_LIST = [
        2019,
        2018,
        2017,
        2016,
        2015,
        2014,
        2013,
        2012,
        2011,
        2010,
        2009
    ]
    # All available geographies
    GEOTYPE_LIST = (
        "nationwide",
        "regions",
        "divisions",
        "states",
        "congressional_districts",
        "state_legislative_upper_districts",
        "state_legislative_lower_districts",
        "counties",
        "places",
        "urban_areas",
        "msas",
        "csas",
        "pumas",
        "nectas",
        "cnectas",
        "aiannh_homelands",
        "tracts",
        "zctas",
        "unified_school_districts",
        "elementary_school_districts",
        "secondary_school_districts",
        "alaska_native",
        "county_subdivision"
    )

    def __init__(
        self,
        api_key=None,
        source="acs5",
        years=None,
        data_dir=None,
        force=False
    ):
        """
        Configuration.
        """
        # Set the inputs
        self.CENSUS_API_KEY = os.getenv("CENSUS_API_KEY", api_key)
        if not self.CENSUS_API_KEY:
            raise NotImplementedError("Census API key required. Pass it as the first argument.")
        self.source = source
        self.force = force

        #
        # Allow custom years for data download, defaulting to most recent year
        #

        # If they want all the years, give it to them.
        if years == "all":
            self.years_to_download = self.YEAR_LIST
        # If the user provides a year give them that.
        elif isinstance(years, int):
            self.years_to_download = [years]
        # Or if they provide years as a list, give those then.
        elif isinstance(years, list):
            self.years_to_download = list(map(int, years))
        # If they provided nothing, default to the latest year of data
        elif years is None:
            self.years_to_download = [max(self.YEAR_LIST), ]

        # Validate the years
        for year in self.years_to_download:
            if year not in self.YEAR_LIST:
                error_msg = ("Data only available for the years"
                             f"{self.YEAR_LIST[-1]}-{self.YEAR_LIST[0]}.")
                raise NotImplementedError(error_msg)

        # Set the data directories
        if data_dir:
            self.data_dir = pathlib.Path(str(data_dir))
        else:
            self.data_dir = self.PARENT_DIR.joinpath("data")
        self.raw_data_dir = self.data_dir.joinpath("raw")
        self.processed_data_dir = self.data_dir.joinpath("processed")

        # Make sure they exist
        if not self.data_dir.exists():
            self.data_dir.mkdir()
        if not self.raw_data_dir.exists():
            self.raw_data_dir.mkdir()
        if not self.processed_data_dir.exists():
            self.processed_data_dir.mkdir()

    @property
    def censusreporter_url(self):
        """
        Returns the URL of the Census Reporter page explaining the ACS table.
        """
        return f"https://censusreporter.org/tables/{self.RAW_TABLE_NAME}/"

    #
    # Geotype downloaders
    #

    @decorators.downloader
    def download_nationwide(self):
        """
        Download nationwide data.
        """
        return geotypes.NationwideDownloader

    @decorators.downloader
    def download_regions(self):
        """
        Download data for all regions.
        """
        return geotypes.RegionsDownloader

    @decorators.downloader
    def download_divisions(self):
        """
        Download data for all divisions.
        """
        return geotypes.DivisionsDownloader

    @decorators.downloader
    def download_states(self):
        """
        Download data for all states.
        """
        return geotypes.StatesDownloader

    @decorators.downloader
    def download_congressional_districts(self):
        """
        Download data for all Congressional districts.
        """
        return geotypes.CongressionalDistrictsDownloader

    @decorators.downloader
    def download_state_legislative_upper_districts(self):
        """
        Download data for all Census upper legislative districts in the provided state.
        """
        return geotypes.StateLegislativeUpperDistrictsDownloader

    @decorators.downloader
    def download_state_legislative_lower_districts(self):
        """
        Download data for all Census lower legislative districts in the provided state.
        """
        return geotypes.StateLegislativeLowerDistrictsDownloader

    @decorators.downloader
    def download_counties(self):
        """
        Download data for all counties.
        """
        return geotypes.CountiesDownloader

    @decorators.downloader
    def download_places(self):
        """
        Download data for all Census designated places.
        """
        return geotypes.PlacesDownloader

    @decorators.downloader
    def download_urban_areas(self):
        """
        Download data for all urban areas
        """
        return geotypes.UrbanAreasDownloader

    @decorators.downloader
    def download_msas(self):
        """
        Download data for Metropolitian Statistical Areas.
        """
        return geotypes.MsasDownloader

    @decorators.downloader
    def download_csas(self):
        """
        Download data for Combined Statistical Areas.
        """
        return geotypes.CsasDownloader

    @decorators.downloader
    def download_pumas(self):
        """
        Download data for Public Use Microdata Areas.
        """
        return geotypes.PumasDownloader

    @decorators.downloader
    def download_nectas(self):
        """
        Download data for New England cities and towns.
        """
        return geotypes.NectasDownloader

    @decorators.downloader
    def download_cnectas(self):
        """
        Download data for combined New England cities and towns.
        """
        return geotypes.CnectasDownloader

    @decorators.downloader
    def download_aiannh_homelands(self):
        """
        Download data for American Indian home lands.
        """
        return geotypes.AiannhHomelandsDownloader

    @decorators.downloader
    def download_tracts(self):
        """
        Download data for all Census tracts in the provided state.
        """
        return geotypes.TractsDownloader

    @decorators.downloader
    def download_zctas(self):
        """
        Download data for Zip Code Tabulation Areas
        """
        return geotypes.ZctasDownloader

    @decorators.downloader
    def download_unified_school_districts(self):
        """
        Download data for unified school districts.
        """
        return geotypes.UnifiedSchoolDistrictsDownloader

    @decorators.downloader
    def download_elementary_school_districts(self):
        """
        Download data for elementary school districts.
        """
        return geotypes.ElementarySchoolDistrictsDownloader

    @decorators.downloader
    def download_secondary_school_districts(self):
        """
        Download data for secondary school districts.
        """
        return geotypes.SecondarySchoolDistrictsDownloader

    @decorators.downloader
    def download_alaska_native(self):
        """
        Download data for Alaska Native regional corporations.
        """
        return geotypes.AlaskaNativeDownloader

    @decorators.downloader
    def download_county_subdivision(self):
        """
        Download data for county subdivisions.
        """
        return geotypes.CountySubdivisionDownloader

    def download_everything(self):
        """
        Download 'em all.
        """
        for geo in self.GEOTYPE_LIST:
            print(geo)
            # Get the downloader function
            dl = getattr(self, f"download_{geo}", None)
            # Validate it
            if not dl or not callable(dl):
                raise NotImplementedError(f"Invalid geography type: {geo}")
            # Run it
            try:
                dl()
            except NotImplementedError:
                pass



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from census_data_downloader.core.tables import BaseTableConfig
from census_data_downloader.core.decorators import register


@register
class DisabilityDownloader(BaseTableConfig):
    """
    Sex by age by disability status.
    https://factfinder.census.gov/help/en/civilian_noninstitutionalized_population.htm
    """
    PROCESSED_TABLE_NAME = "disability"
    UNIVERSE = "civilian noninstitutionalized population"
    RAW_TABLE_NAME = 'B18101'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': "universe",
        '002': "male",
        '003': "male_under_5",
        '004': "male_under5_disability",
        '005': "male_under5_nodisability",
        '006': "male_5to17",
        '007': "male_5to17_disability",
        '008': "male_5to17_nodisability",
        '009': "male_18to34",
        '010': "male_18to34_disability",
        '011': "male_18to34_nodisability",
        '012': "male_35to64",
        '013': "male_35to64_disability",
        '014': "male_35to64_nodisability",
        '015': "male_65to74",
        '016': "male_65to74_disability",
        '017': "male_65to74_nodisability",
        '018': "male_75older",
        '019': "male_75older_disability",
        '020': "male_75older_nodisability",
        '021': "female",
        '022': "female_under_5",
        '023': "female_under5_disability",
        '024': "female_under5_nodisability",
        '025': "female_5to17",
        '026': "female_5to17_disability",
        '027': "female_5to17_nodisability",
        '028': "female_18to34",
        '029': "female_18to34_disability",
        '030': "female_18to34_nodisability",
        '031': "female_35to64",
        '032': "female_35to64_disability",
        '033': "female_35to64_nodisability",
        '034': "female_65to74",
        '035': "female_65to74_disability",
        '036': "female_65to74_nodisability",
        '037': "female_75older",
        '038': "female_75older_disability",
        '039': "female_75older_nodisability",
    })


@register
class DisabilityPovertyDownloader(BaseTableConfig):
    """
    Age by disability status by poverty status. Income in the past 12 months.
    https://factfinder.census.gov/help/en/civilian_noninstitutionalized_population.htm
    """
    PROCESSED_TABLE_NAME = "disabilitypoverty"
    UNIVERSE = "civilian noninstitutionalized population for whom poverty status is determined"
    RAW_TABLE_NAME = 'C18130'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': "universe",
        '002': "under18",
        '003': "under18_disability",
        '004': "under18_disability_below_povertylevel",
        '005': "under18_disability_atorabove_povertylevel",
        '006': "under18_nodisability",
        '007': "under18_nodisability_below_povertylevel",
        '008': "under18_nodisability_atorabove_povertylevel",
        '009': "18to64",
        '010': "18to64_disability",
        '011': "18to64_disability_below_povertylevel",
        '012': "18to64_disability_atorabove_povertylevel",
        '013': "18to64_nodisability",
        '014': "18to64_nodisability_below_povertylevel",
        '015': "18to64_nodisability_atorabove_povertylevel",
        '016': "65older",
        '017': "65older_disability",
        '018': "65older_disability_below_povertylevel",
        '019': "65older_disability_atorabove_povertylevel",
        '020': "65older_nodisability",
        '021': "65older_nodisability_below_povertylevel",
        '022': "65older_nodisability_atorabove_povertylevel",
    })


@register
class CognitiveDifficultyDownloader(BaseTableConfig):
    """
    Sex by age by cognitive difficulty.
    https://factfinder.census.gov/help/en/civilian_noninstitutionalized_population.htm
    """
    PROCESSED_TABLE_NAME = "cognitivedifficulty"
    UNIVERSE = "civilian noninstitutionalized population 5 years or over"
    RAW_TABLE_NAME = 'B18104'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': "universe",
        '002': "male",
        '003': "male_5_17",
        '004': "male_5_17_cognitive",
        '005': "male_5_17_nocognitive",
        '006': "male_18_34",
        '007': "male_18_34_cognitive",
        '008': "male_18_34_nocognitive",
        '009': "male_35_64",
        '010': "male_35_64_cognitive",
        '011': "male_35_64_nocognitive",
        '012': "male_65_74",
        '013': "male_65_74_cognitive",
        '014': "male_65_74_nocognitive",
        '015': "male_75_older",
        '016': "male_75_older_cognitive",
        '017': "male_75_older_nocognitive",
        '018': "female",
        '019': "female_5_17",
        '020': "female_5_17_cognitive",
        '021': "female_5_17_nocognitive",
        '022': "female_18_34",
        '023': "female_18_34_cognitive",
        '024': "female_18_34_nocognitive",
        '025': "female_35_64",
        '026': "female_35_64_cognitive",
        '027': "female_35_64_nocognitive",
        '028': "female_65_74",
        '029': "female_65_74_cognitive",
        '030': "female_65_74_nocognitive",
        '031': "female_75_older",
        '032': "female_75_older_cognitive",
        '033': "female_75_older_nocognitive",
    })



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from census_data_downloader.core.tables import BaseTableConfig
from census_data_downloader.core.decorators import register


@register
class ClassOfWorkerDownloader(BaseTableConfig):
    """
    Sex by Class of Worker for the Civilian Employed Population 16 Years and Over.
    """
    PROCESSED_TABLE_NAME = "classofworker"
    UNIVERSE = "civilian employed population 16 years and over"
    RAW_TABLE_NAME = 'B24080'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': "universe",
        '002': "male_total",
        '003': "male_private_for_profit_wage_and_salary",
        '004': "male_employee_of_private_company",
        '005': "male_selfemployed_in_own_incorporated_business",
        '006': "male_private_not_for_profit_wage_and_salary",
        '007': "male_local_government",
        '008': "male_state_government",
        '009': "male_federal_government",
        '010': "male_selfemployed_in_own_not_incorporated_business",
        '011': "male_unpaid_family_workers",
        '012': "female_total",
        '013': "female_private_for_profit_wage_and_salary",
        '014': "female_employee_of_private_company",
        '015': "female_selfemployed_in_own_incorporated_business",
        '016': "female_private_not_for_profit_wage_and_salary",
        '017': "female_local_government",
        '018': "female_state_government",
        '019': "female_federal_government",
        '020': "female_selfemployed_in_own_not_incorporated_business",
        '021': "female_unpaid_family_workers"
    })



#! /usr/bin/env python
# -*- coding: utf-8 -*
import collections
from census_data_downloader.core.tables import BaseTableConfig
from census_data_downloader.core.decorators import register


@register
class SnapbyPovertyDownloader(BaseTableConfig):
    """
    receipt of food stamps/snap in past 12 months by poverty status in the past 12 months for households
    """
    PROCESSED_TABLE_NAME = "snap"
    UNIVERSE = "households"
    RAW_TABLE_NAME = 'B22003'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': "universe",
        '002': "received_snap_in_past_12_months",
        '003': "received_snap_in_past_12_months_income_below_poverty",
        '004': "received_snap_in_past_12_months_income_atorabove_poverty",
        '005': "not_received_snap_in_past_12_months",
        '006': "not_received_snap_in_past_12_months_income_below_poverty",
        '007': "not_received_snap_in_past_12_months_income_atorabove_poverty",
    })


@register
class SnapLatinoDownloader(BaseTableConfig):
    """
    receipt of food stamps/snap in past 12 months by race of householder
    """
    PROCESSED_TABLE_NAME = "snaplatino"
    RAW_TABLE_NAME = 'B22005I'
    RAW_FIELD_CROSSWALK = collections.OrderedDict({
        '001': "universe",
        '002': "received_snap_in_past_12_months",
        '003': "not_received_snap_in_past_12_months"
    })


@register
class SnapWhiteDownloader(SnapLatinoDownloader):
    PROCESSED_TABLE_NAME = "snapwhite"
    RAW_TABLE_NAME = 'B22005H'


@register
class SnapBlackDownloader(SnapLatinoDownloader):
    PROCESSED_TABLE_NAME = "snapblack"
    RAW_TABLE_NAME = 'B22005B'


@register
class SnapAsianDownloader(SnapLatinoDownloader):
    PROCESSED_TABLE_NAME = "snapasian"
    RAW_TABLE_NAME = 'B22005D'



from os import path
from setuptools import setup

# Read the contents of the README file
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()
    
setup(
    name='census-data-downloader',
    version='0.0.32',
    description="Download U.S. census data and reformat it for humans",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Los Angeles Times Data and Graphics Department',
    author_email='datadesk@latimes.com',
    url='http://www.github.com/datadesk/census-data-downloader',
    license="MIT",
    packages=(
        "census_data_downloader",
        "census_data_downloader.core",
        "census_data_downloader.tables",
    ),
    install_requires=(
        "pandas",
        "us",
        "census",
        "census-data-aggregator",
        "click",
        "jinja2"
    ),
    entry_points="""
        [console_scripts]
        censusdatadownloader=census_data_downloader.cli:cmd
    """,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'License :: OSI Approved :: MIT License',
    ],
)
